require 'gosu'
require 'ruby_man/resource_manager'
require 'ruby_man/level_loader'
require 'ruby_man/GameObjects/menu'
require 'ruby_man/GameObjects/game_over'

module RubyMan
  # Game window and the game itself
  class GameMain < Gosu::Window
    include LevelLoader

    attr_reader :x_res, :y_res
    attr_accessor :lives, :score
    attr_reader :scores
    attr_accessor :player
    attr_reader :objects, :moving_objects

    def initialize
      @x_res, @y_res = 800, 640
      super(@x_res, @y_res, false)
      @resmgr = ResourceManager.new(self)
      @objects, @moving_objects = [], []
      @last_milliseconds = Gosu.milliseconds
      @food_left = 0
      @pause_time = -0.5
      self.caption = 'RubyMan'
      load_scores
      load_level_list
      main_menu
    end

    def load_scores
      @config_file_name = "#{Dir.home}/.rubyman.cfg"
      @scores = []
      @score = 0
      return unless File.exist?(@config_file_name)
      File.open(@config_file_name).each do |line|
        entry = line.strip.split(';')
        entry[2] = entry[2].to_i
        @scores.push(entry)
      end
    end

    def save_scores
      file = File.open("#{Dir.home}/.rubyman.cfg", 'w')
      count = 0
      @scores.each do |entry|
        count += 1
        break if count > 10
        line = "#{entry[0]};#{entry[1]};#{entry[2]}"
        file.puts(line)
      end
    end

    def add_score(name)
      entry = [name, Time.new.strftime('%Y-%m-%d %H:%M'), @score]
      @scores.push(entry)
      sort_scores
      save_scores
    end

    def sort_scores
      @scores.sort! do |a, b|
        a[2] <=> b[2]
      end
      @scores.reverse!
      @scores = @scores.first(10)
    end

    def load_level_list
      @levels = []
      @level = 0
      File.open(ResourceManager.media('levellist.txt')).each do |line|
        @levels.push(line.strip)
      end
    end

    def start
      @lives = 3
      @score = 0
      @level = -1
      next_level
    end

    def restart
      persistent = @objects.select { |obj| obj.class.persistent }
      persistent_moving = @moving_objects.select { |obj| obj.class.persistent }
      load_level(@level, true)
      @objects += persistent
      @moving_objects += persistent_moving
      pause(1000)
    end

    def next_level
      if @level < @levels.size - 1
        @level += 1
        load_level(@level, false)
        ResourceManager['sfx_begin'].play
        pause(3000)
      else
        game_over
      end
    end

    def end_level
      @objects.clear
      @moving_objects.clear
    end

    def load_level(num, restart)
      end_level
      load_level_file(@levels[num], restart)
    end

    def level_loaded
      # reevaluate ghost directions after level is fully loaded
      moving_objects.each { |obj| obj.select_direction! if obj.is_a?(Ghost) }
    end

    def game_over
      end_level
      add_object(GameOver.new)
    end

    def main_menu
      end_level
      add_object(Menu.new)
    end

    def add_object(obj)
      @objects.push(obj)
      obj.game = self
      @moving_objects.push(obj) if obj.is_a?(MovingObject)
      obj.created
    end

    def destroy_object(obj)
      obj.destroyed
      @objects.delete(obj)
      @moving_objects.delete(obj) if obj.is_a?(MovingObject)
    end

    def button_down(id)
      close if id == Gosu::KbEscape
      pause if id == Gosu::KbSpace
      @objects.each { |obj| obj.button_down(id) }
    end

    def pause(ms = -1000.0)
      if paused
        @pause_time = -0.5
      else
        @pause_time = ms / 1000.0
      end
    end

    def paused
      return true if @pause_time > 0 || @pause_time == -1
      false
    end

    def update
      current_time = Gosu.milliseconds / 1000.0
      delta = [current_time - @last_milliseconds, 0.25].min
      @last_milliseconds = current_time

      update_delta(delta)
    end

    def update_delta(delta)
      @pause_time -= delta if @pause_time > 0

      return if paused
      @objects.each { |obj| obj.update(delta) }
      check_collisions
    end

    def draw
      super
      @objects.each(&:draw)
    end

    def check_collisions
      @moving_objects.each do |obj1|
        bbox = obj1.bbox
        @moving_objects.each do |obj2|
          obj2.collision(obj1) if bbox.intersects?(obj2.bbox)
        end
      end
    end

    def place_free(x, y)
      objects_at(x, y).each { |obj| return false if obj.solid }
      true
    end

    def objects_at(x, y)
      @objects.select do |obj|
        next unless obj.is_a?(GridObject)
        obj.grid_x == x && obj.grid_y == y
      end
    end

    def food_created
      @food_left += 1
    end

    def food_destroyed
      @food_left -= 1
      next_level if @food_left == 0
    end
  end
end
