require 'gosu'
require 'ruby_man/sprite'

module Gosu
  # extension of Font for easier centered drawing
  class Font
    def draw_centered(text, x, y, z = 0,
                      factor_x = 1, factor_y = 1,
                      color = 0xffffffff, mode = :default)
      w = text_width(text, factor_x)
      draw(text, x - w / 2, y, z, factor_x, factor_y, color, mode)
    end
  end
end

module RubyMan
  # Handles loading of resources for faster and centralized access.
  class ResourceManager
    class << self
      attr_accessor :inst
    end

    def initialize(window)
      ResourceManager.inst = self
      @window = window
      @resources = {}
      @media_dir = "#{__dir__}/../../media"
      load_config
    end

    def media_file_path(media_path)
      "#{@media_dir}/#{media_path}"
    end

    def load_config
      File.open(media_file_path('resource.cfg')).each do |line|
        cfg = line.split(';').map!(&:strip)
        next unless cfg.size >= 3
        case cfg[1]
        when 'image' then load_image(cfg)
        when 'animation' then load_animation(cfg)
        when 'font' then load_font(cfg)
        when 'sound' then load_sound(cfg)
        end
      end
    end

    def load_animation(cfg)
      return unless cfg.size >= 7

      sprite = Sprite.new
      x_offset = cfg[3].to_i
      y_offset = cfg[4].to_i
      sprite.width = cfg[5].to_i
      sprite.height = cfg[6].to_i
      frames  = cfg[7].to_i

      (0...frames).each do |i|
        image = Gosu::Image.new(@window, media_file_path(cfg[2]), true,
                                x_offset + i * sprite.width, y_offset,
                                sprite.width, sprite.height)
        sprite.frames.push(image)
      end

      @resources[cfg[0]] = sprite
    end

    def load_image(cfg)
      sprite = Sprite.new
      sprite.frames.push(Gosu::Image.new(@window, media_file_path(cfg[2]), false))
      sprite.width = sprite.frames[0].width
      sprite.height = sprite.frames[0].height
      @resources[cfg[0]] = sprite
    end

    def load_sound(cfg)
      sound = Gosu::Sample.new(@window, media_file_path(cfg[2]))
      @resources[cfg[0]] = sound
    end

    def load_font(cfg)
      begin
        font = Gosu::Font.new(@window, media_file_path(cfg[2]), cfg[3].to_i)
        font.text_width('font found test')
      rescue
        font = Gosu::Font.new(@window, Gosu::default_font_name, cfg[3].to_i)
      end
      @resources[cfg[0]] = font
    end

    def resource(name)
      @resources[name]
    end

    def self.[](resource_name)
      @inst.resource(resource_name)
    end

    def self.media(media_path)
      @inst.media_file_path(media_path)
    end
  end
end
